@@ -31,6 +31,11 @@
 #define	NODIE	0
 #define	DIE	1
 
+/* Note for porters: see the PORTING file if your compiler
+   does not support unsigned long long.
+*/
+typedef  unsigned long long ulonglong;
+
 /* KH */
 #define MEMFD 10000
 #define ZIPFD 10001
@@ -82,13 +87,20 @@ extern int zipfdfd;
  * Swap bytes.
  */
 #define	swab(n)		\
-	((((ushort)(n) >> 8) & 0xff) | (((ushort)(n) << 8) & 0xff00))
+	((((unsigned short)(n) >> 8) & 0xff) | (((unsigned short)(n) << 8) & 0xff00))
 
 /*
  * Cast and reduce to unsigned short.
  */
 #define	ush(n)		\
-	(((ushort) (n)) & 0177777)
+	(((unsigned short) (n)) & 0177777)
+
+#define ulo(n) 		\
+	(((unsigned long) (n)) & 0xFFFFFFFFL)
+
+#define ull(n)		\
+	(((unsigned long long) (n)) & 0xFFFFFFFFFFFFFFFFLL)
+/*                             1234567890123456 */
 
 /*
  * Definitions.
@@ -96,14 +108,21 @@ extern int zipfdfd;
 #define	reg	register	/* Convenience */
 #define	uint	unsigned int	/* Not always in types.h */
 #define	ushort	unsigned short	/* Not always in types.h */
+
 #define	BLOCK	5120		/* Default archive block size */
 #define	FSBUF	(8*1024)	/* Filesystem buffer size */
 #define	H_COUNT	10		/* Number of items in ASCII header */
 
-/* old ASCII format */
-#define	H_PRINT	"%06o%06o%06o%06o%06o%06o%06o%011lo%06o%011lo"
+/* binary format */
+#define	M_BINARY 070707		/* Binary magic number */
+#define	M_STRLEN 6		/* ASCII magic number length */
+
+/* (old) ASCII format */
+#define	H_STRLEN 70		/* old ASCII header string length */
+#define	M_ASCII "070707"	/* old ASCII magic number */
+#define H_PRINT "%06o%06o%06o%06o%06o%06o%06o%011lo%06o%011lo"
 /* H_SCAN is obsolete, replaced by PH_SCAN to be more portable. */
-#define	H_SCAN	"%6ho%6ho%6ho%6ho%6ho%6ho%6ho%11lo%6o%11lo"
+#define H_SCAN  "%6ho%6ho%6ho%6ho%6ho%6ho%6ho%11lo%6o%11lo"
 #define PH_SCAN  "%6lo%6lo%6lo%6lo%6lo%6lo%6lo%11lo%6o%11lo"
 
 /*
@@ -112,16 +131,32 @@ ASCII format headers.
 
 |23456|23456|23456|23456|23456|23456|23456|23456|23456789ab|23456|23456789ab|-
 |  hdr|  dev|  ino| mode|  uid|  gid|nlink| rdev|     mtime|nmlen|      size|n
-
-
 */
 
-
 /* extended ASCII format */
-#define	H_PRINT2  "%06o%011lo%06o%06o%06o%06o%06o%011lo%06o%011lo"
+#define	H_STRLEN2 75		/* extended ASCII header string length */
+#define	M_ASCII2 "070717"	/* extended ASCII magic number */
+#define H_PRINT2  "%06o%011lo%06o%06o%06o%06o%06o%011lo%06o%011lo"
 #define PH_SCAN2  "%6lo%11lo%6lo%6lo%6lo%6lo%6lo%11lo%6o%11lo"
 
 
+/*
+Below is a handy piece of ASCII art which can be used to decode large
+ASCII format headers.
+
+|23456|2345678|234567890123456m|23456|2345678|2345678|2345678|2345678|234567890123456n|234|234|234s|234567890123456:|-
+|  hdr|    dev|            inoM|  mod|    uid|    gid|  nlink|   rdev|          mtimeN|nml|flg|xszS|           size:|n
+*/
+
+/* large ASCII format */
+#define	H_STRLEN3 110		/* large ASCII string length */
+#define	M_ASCII3 "070727"	/* large ASCII magic number */
+#define H_COUNT3 12
+#define	H_PRINT3  "%08lX%016llXm%06lo%08lX%08lX%08lX%08lX%016llXn%04X%04X%04Xs%016llX:"
+#define	PH_SCAN3  "%8lX%16llXm%6lo%8lX%8lX%8lX%8llX%16llXn%4X%4X%4Xs%16llX:"
+
+
+
 typedef struct {
 long unsigned int PSt_dev;
 long unsigned int PSt_ino;
@@ -134,12 +169,21 @@ long unsigned int PSt_mtime;
 long unsigned int PSt_size;
 } PStat;
 
-#define	H_STRLEN 70		/* old ASCII header string length */
-#define	M_ASCII "070707"	/* old ASCII magic number */
-#define	H_STRLEN2 75		/* extended ASCII header string length */
-#define	M_ASCII2 "070717"	/* extended ASCII magic number */
-#define	M_BINARY 070707		/* Binary magic number */
-#define	M_STRLEN 6		/* ASCII magic number length */
+
+typedef struct {
+long unsigned int PSt_dev;
+unsigned long long PSt_ino;
+long unsigned int PSt_mode;
+long unsigned int PSt_uid;
+long unsigned int PSt_gid;
+long unsigned int PSt_nlink;
+unsigned long long PSt_rdev;
+unsigned long long PSt_mtime;
+unsigned long long PSt_size;
+} PHStat;
+
+
+
 #define	NULLDEV	-1		/* Null device code */
 #define	NULLINO	0		/* Null inode number */
 #define	PATHELEM 256		/* Pathname element count limit */
@@ -152,6 +196,15 @@ long unsigned int PSt_size;
 #define	STDOUT	1		/* Standard output file descriptor */
 #define	TTY	"/dev/tty"	/* For volume-change queries */
 
+/* the three flags below are used in the match.c code, they allow us
+   to store 3 types of patterns in a single `all patterns' linked
+   list.  (the use of a single list is for historical reasons, it is
+   not the most ovbious design for the current needs)
+ */
+#define PATTYPE_MATCH 0 
+#define PATTYPE_NOMATCH 1
+#define PATTYPE_EXT 2
+
 #ifndef PRG_COMPRESS
 #define PRG_COMPRESS "compress"
 #endif
@@ -227,6 +280,7 @@ typedef struct
 {
   struct stat sb_stat;
   char sb_link[PATHSIZE];
+  ino_t ino_orig; /* used in -o mode to support -ovv */
 } Stat;
 
 #define	STAT(name, asb)		stat(name, &(asb)->sb_stat)
@@ -328,117 +382,129 @@ typedef struct link
   Path *l_path;			/* Pathname(s) to link from */
 } Link;
 
+/*
+ * Directory information. One entry exists for each directory
+ * in order to update it with the correct timestamps.
+ */
+typedef struct dir
+{
+  struct dir *d_forw;		/* Forward chain (terminated) */
+  time_t d_mtime;		/* Modification time */
+  char *d_name;			/* Pathname of directory */
+} Dir;
+ 
+
 
 /*
  * Internal functions.
  */
-VOIDFN copyin ();
-VOIDFN copyout ();
-void compressfile ();
-int dirchg ();
-int dirmake ();
-int dirneed ();
-void fatal ();
-void goodbye ();
-VOIDFN in ();
-void inalloc ();
-int inascii ();
-int inavail ();
-int inbinary ();
-int indata ();
-int inentry ();
-int infill ();
-int inhead ();
-int inhead2 ();
-int inread ();
-int inskip ();
-int inswab ();
-int lineget ();
-void linkalso ();
-Link *linkfrom ();
-void linkleft ();
-Link *linkto ();
+VOIDFN copyin (char **);
+VOIDFN copyout (char **);
+void compressfile (int *, char *, Stat *, int *);
+int dirchg (char *, char *);
+int dirmake (char *, Stat *);
+int dirneed (char *);
+void fatal (char *,char *);
+void goodbye (int);
+VOIDFN in (char **);
+void inalloc (uint);
+int inascii (char *, char *, Stat *);
+int inascii2 (char *, char *, Stat *);
+int inascii3 (char *, char *, Stat *);
+int inavail (char **, uint*);
+int inbinary (char *, char *, Stat *);
+int indata (int, off_t, char*);
+int inentry (char *, Stat *);
+int infill (void);
+int inhead (char *, Stat *);
+int inread (char *, uint);
+int inskip (off_t);
+int inswab (char *, char *, Stat *);
+int lineget (FILE *, char *, int);
+void linkalso (Link *, char * );
+Link *linkfrom (Stat *, int);
+void linkleft (void);
+Link *linkto (char *, Stat *);
 #ifndef MEMCPY
-char *memcpy ();
+char *memcpy (char *, char *, uint);
 #endif
-char *memget ();
-char *memstr ();
+char *memget (uint);
+char *memstr (char *);
 #ifndef MKDIR
-int mkdir ();
+int mkdir (char *, ushort);
 #endif
-void nameadd ();
-int namecmp ();
-int nameopt ();
-void next ();
-void nextask ();
-void nextclos ();
-int nextopen ();
-int openin ();
-int openotty ();
-int openqtty ();
-int options ();
-VOIDFN out ();
-void outalloc ();
-uint outavail ();
-int outdata ();
+void nameadd (char *, int);
+int namecmp (char *, Stat *);
+int namecmp_ext (char *);
+int nameopt (char *);
+void next (int, char *);
+void nextask (char *, char *, int);
+void nextclos (void );
+int nextopen (int);
+int openin (char *, char *, Stat *, int *);
+int openotty (char *, Stat *, Link *, int, int);
+int openqtty (void);
+int options (int, char **, char *);
+VOIDFN out (char **);
+void outalloc (uint);
+uint outavail (char **);
+int outdata (int, char*, off_t);
 
-void outdatazip (); /* added KH */
-void handler();     /* added KH */
-void waitforgzip();     /* added KH */
-void outdatamem (); /* added KH */
-void memreset(); /* added KH */
+void outdatazip (int, char*, off_t); /* added KH */
+void waitforgzip(void);     /* added KH */
+void outdatamem (char *, off_t); /* added KH */
+void memreset(void); /* added KH */
 int memread(char *buf,int count); /* added KH */
-void memfree(); /* added KH */
-int readcompexts(); /* added KH */
-int nameaddfile(); /* added KH */
+void memfree(void); /* added KH */
+int nameaddfile(char *, int); /* added KH */
 
-void outeof ();
-void outflush ();
-void outhead ();
-void outhead2 ();
-void outpad ();
-void outwait ();
-void outwrite ();
-VOIDFN pass ();
-void passdata ();
-int passitem ();
-int pipechld ();
-int pipeopen ();
-void pipewait ();
-void prsize ();
-VOIDFN readcheck ();
+void outeof (char *, uint);
+void outflush (int);
+void outhead (char *, Stat *);
+void outhead2 (char *, Stat *);
+void outhead3 (char *, Stat *);
+void outpad (off_t);
+void outwait (void);
+void outwrite (char *,uint);
+VOIDFN pass (char **);
+void passdata (char *, int, char*, int);
+int passitem (char *, Stat *,int, char**);
+int pipechld (int, int *);
+int pipeopen (int );
+void pipewait (void);
+void prsize (FILE *, ulonglong);
+VOIDFN readcheck (char **);
 #ifndef MKDIR
-int rmdir ();
+int rmdir (char *);
 #endif
-#if !defined (linux) && !defined(__FreeBSD__)
+#if !defined (linux) && !defined(__FreeBSD__) && !defined(sun)
 VOIDFN (*signal ())();
 #endif
-     int fswrite ();
-#ifdef USESHMEM
-     char *shmemalloc ();
-     void shmemfree ();
-#endif
-     char *syserr ();
-     VOIDFN toc ();
-     void tocentry ();
-     void tocmode ();
-     void usage ();
-     void verify ();
-     int warn ();
-     int warn_nocount ();
-     int warnarch ();
-     int writedisk ();
-     int xfork ();
-     void xpause ();
-     int xwait ();
+     int fswrite (int, char*, uint);
+     char *syserr (void);
+     VOIDFN toc (char **);
+     void tocentry (char *, Stat *);
+     void tocmode (mode_t);
+     void usage (void);
+     void verify (int);
+     int warn (char *, char *);
+     int warn_nocount (char*, char *);
+     int warnarch (char *, off_t);
+     int writedisk (int);
+     int xfork (char *, int);
+     void xpause (void);
+     int xwait (int, char *, int);
      void mail(char *who,int vol,char *archive);
 
-int writeall();
-int incheckentry();
-int openincheck();
-void mayberewind();
+int writeall(int, const char*, unsigned int);
+int incheckentry(char *, Stat *);
+int incheckdata (int fd, off_t size, char *name, Stat *asb, int comp);
+Link * linkinode16 (dev_t dev, ino_t ino);
+int opencontrolscript (char *name);
+
+int openincheck(char *, Stat *, int *,int);
+void mayberewind(void);
 
-extern	void clear_args(void);
 extern  void add_arg(char *arg);
 extern  char *compress_arg_list[];
 
@@ -451,6 +517,7 @@ extern long compthreshold;
 extern int ignoreslash;
 extern short Zflag;
 extern int arfd;
+extern int extcasesens;
 
 extern int forceZflag;
 extern char *compressprog;
@@ -458,45 +525,8 @@ extern int compressargs;
 extern int rewindfd;
 extern char *ignorewarnings;
 
-/* Note for porters: we use the ulonglong type in calculations with
-   integers which should be large enough to hold the size of 1 tape
-   (if -s option used) or the size of the entire archive.  With >2GB
-   tapes appearing, the off_t type (32 bit signed on Linux), which was
-   previously used, was getting too small.
+extern int readcompexts(char*);
 
-   long long is supported by many compilers as a >=64 bit integer, so
-   we use it to ensure correct byte position reporting and tape change
-   operations for >2GB tapes.
-
-   Alternatives in the ifs below: double is 64 bit floating point on
-   many compilers, which gives us enough bits in the mantissa part to
-   go to really large (2GB>) integer numbers without loss of accuracy.
-   If double is 32 bits then it will loose accuracy to soon and a
-   plain long (assuming plain long is 32 bits) will have to be used to
-   go to 2GB reliably.  A long double would be another option, if that
-   is bigger than a double.  
-*/
-
-#if 1
-
-typedef  unsigned long long ulonglong;
-
-#else
-
-#if 1
-
-typedef double ulonglong;
-/* also need to change a % into an fmod() somewhere in afio.c
-   if we use this one. */
-
-#else
-
-typedef  unsigned long ulonglong;
-
-#endif
-
-#endif
-
-ulonglong optsize ();
-void update_aruntil();
+ulonglong optsize (char *);
+void update_aruntil(void);
 extern ulonglong maxsizetocompress;
