@@ -15,6 +15,9 @@
 #include "afio.h"
 
 int matchcompext(char *);
+void meminit(void);
+void memwrite(char *buf, int count);
+int setupgzip(char *name);
 
 int zipfdfd;
 
@@ -71,7 +74,7 @@ char *membank=NULL;
 size_t allocsize=0;
 size_t memsize,membytesread,membytesleft;
 
-void meminit()
+void meminit(void)
 {
  memerror=0;
  memsize=0;
@@ -111,7 +114,7 @@ void memwrite(char *buf, int count)
     }
  }
 
- memcpy(membank+memsize,buf,count);
+ memcpy(membank+memsize,buf,(size_t)count);
  memsize+=count;
 }
 
@@ -125,8 +128,8 @@ int memread(char *buf,int count)
 {
  if(membytesleft==0) return 0;
 
- if (count > membytesleft) count=membytesleft;
- memcpy(buf,membank+membytesread,count);
+ if ((size_t)count > membytesleft) count=membytesleft;
+ memcpy(buf,membank+membytesread,(size_t)count);
  membytesread+=count;
  membytesleft-=count;
  return count; 
@@ -214,18 +217,14 @@ void waitforgzip()
 #endif
 
 void
-compressfile (fdp, name, asb, cratio)
-     int *fdp;
-     char *name;
-     reg Stat *asb;
-     int *cratio;
-{
+compressfile (int *fdp, char *name, reg Stat *asb, int *cratio) {
   int compout;
   char *tmpcomp;
   Stat asb2;
-  int zipfd,len;
-  char buf[4096];
-  long ziplen;
+  int zipfd;
+  ssize_t len;
+  char buf[40960];
+  off_t ziplen;
   int namelen,usemem;
 
   if (cratio)
@@ -273,8 +272,8 @@ compressfile (fdp, name, asb, cratio)
        &&
        ( ( !matchcompext(name)
            && (asb->sb_size >= compthreshold) 
-           && ((maxsizetocompress==0) || (asb->sb_size <= maxsizetocompress))
-         )
+           && ((maxsizetocompress==0) || ((ulonglong)(asb->sb_size) <= maxsizetocompress))
+	   )
 	 || forceZflag
        )
      )
@@ -304,7 +303,7 @@ compressfile (fdp, name, asb, cratio)
       {
         ziplen=0;
         if(usemem) meminit();
-        while((len=read(zipfd,buf,4096))!=0)
+        while((len=read(zipfd,buf,sizeof(buf)))!=0)
           {
            if(len<0) {  fprintf(stderr,
                           "Trouble zipping file, storing uncompressed\n"); 
@@ -312,6 +311,7 @@ compressfile (fdp, name, asb, cratio)
                         ziplen=0; break;
                      }    
            if(usemem) memwrite(buf,len);
+
            ziplen+=len;
            /* too much memory used? */
            if(usemem && ziplen>maxmem)
