@@ -1,5 +1,5 @@
-#define VERSION	"2.4.7"
-#define DATE	"9 Oct 2001"
+#define VERSION	"2.5"
+#define DATE	"21 Dec 2003"
 
 /*
  * Version 1.68		               1985	Mark Brukhartz
@@ -18,4 +18,8 @@
  * Version 2.4.5                28 Sep 1998     Koen Holtman
  * Version 2.4.6                25 Nov 1999     Koen Holtman
  * Version 2.4.7                09 Oct 2001     Koen Holtman
+ * Version 2.4.7.9beta          01 Dec 2002     Koen Holtman
+ * Version 2.4.7.9beta4         04 Dec 2002     Koen Holtman
+ * Version 2.4.8.beta1          14 Dec 2003     Koen Holtman
+ * Version 2.5                  21 Dec 2003     Koen Holtman
  */
